Learn R Programming

Matrix (version 1.7-6)

RsparseMatrix-class: Class "RsparseMatrix" of Sparse Matrices in Row-compressed Form

Description

The "RsparseMatrix" class is the virtual class of all sparse matrices coded in sorted compressed row-oriented form. Since it is a virtual class, no objects may be created from it. See showClass("RsparseMatrix") for its subclasses.

Arguments

See Also

its superclass, sparseMatrix, and, e.g., class dgRMatrix for the links to other classes.

Examples

Run this code
showClass("RsparseMatrix")

Run the code above in your browser using DataLab